@@ -345,7 +345,7 @@ static size_t EncodeImage(Image *image,const unsigned long bytes_per_line,
         Determine runlength.
       */
       for (i=1; ((x+i) < (long) bytes_per_line); i++)
-        if ((*(p+i) != *p) || (i == 255))
+        if ((i == 255) || (*(p+i) != *p))
           break;
       *q++=(unsigned char) i;
       *q++=(*p);
@@ -552,15 +552,13 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     bmp_info.offset_bits=ReadBlobLSBLong(image);
     bmp_info.size=ReadBlobLSBLong(image);
     if (logging)
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  BMP size: %lu",
-        bmp_info.size);
-    if (bmp_info.file_size != GetBlobSize(image))
-      ThrowReaderException(CorruptImageError,"LengthAndFilesizeDoNotMatch",
-        image);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "  BMP size: %lu, File size: %lu",
+        bmp_info.size, (unsigned long) GetBlobSize(image));
     if (bmp_info.size == 12)
       {
         /*
-          OS/2 BMP image file.
+          Windows 2.X or OS/2 BMP image file.
         */
         bmp_info.width=(short) ReadBlobLSBShort(image);
         bmp_info.height=(short) ReadBlobLSBShort(image);
@@ -575,9 +573,11 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (logging)
           {
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "  Format: OS/2 Bitmap");
+              "  Format: Windows 2.X or OS/2 Bitmap");
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
               "  Geometry: %ldx%ld",bmp_info.width,bmp_info.height);
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+              "  Bits per pixel: %d",bmp_info.bits_per_pixel);
           }
       }
     else
@@ -759,6 +759,10 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
       }
 
+    if ((bmp_info.compression != BI_RGB) &&
+        (bmp_info.file_size != GetBlobSize(image)))
+      ThrowReaderException(CorruptImageError,"LengthAndFilesizeDoNotMatch",
+        image);
     if (bmp_info.width <= 0)
       ThrowReaderException(CorruptImageWarning,"NegativeOrZeroImageSize",image);
     if (bmp_info.height == 0)
@@ -1096,32 +1100,24 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             pixel=(*p++);
             pixel|=(*p++) << 8;
             red=((pixel & bmp_info.red_mask) << shift.red) >> 16;
+            if (quantum_bits.red == 8)
+              red|=(red >> 8);
             green=((pixel & bmp_info.green_mask) << shift.green) >> 16;
+            if (quantum_bits.green == 8)
+              green|=(green >> 8);
             blue=((pixel & bmp_info.blue_mask) << shift.blue) >> 16;
-            opacity=((pixel & bmp_info.alpha_mask) << shift.opacity) >> 16;
+            if (quantum_bits.blue == 8)
+              blue|=(blue >> 8);
+            if (image->matte != False)
+              {
+                opacity=((pixel & bmp_info.alpha_mask) << shift.opacity) >> 16;
+                if (quantum_bits.opacity == 8)
+                  opacity|=(opacity >> 8);
+                  q->opacity=ScaleShortToQuantum(opacity);
+              }
             q->red=ScaleShortToQuantum(red);
-            if (quantum_bits.red == 5)
-              q->red|=(q->red >> 5);
             q->green=ScaleShortToQuantum(green);
-            if (quantum_bits.green == 5)
-              q->green|=(q->green >> 5);
-            if (quantum_bits.green == 6)
-              q->green|=(q->green >> 6);
             q->blue=ScaleShortToQuantum(blue);
-            if (quantum_bits.blue == 5)
-              q->blue|=(q->blue >> 5);
-            if (image->matte)
-              q->opacity=ScaleShortToQuantum(opacity);
-#if (QuantumDepth > 8)
-            if (quantum_bits.blue <= 8)
-              q->blue|=((q->blue & 0xff00) >> 8);
-            if (quantum_bits.green <= 8)
-              q->green|=((q->green & 0xff00) >> 8);
-            if (quantum_bits.red <= 8)
-              q->red|=((q->red & 0xff00) >> 8);
-            if (quantum_bits.opacity <= 8)
-              q->opacity|=((q->opacity & 0xff00) >> 8);
-#endif
             q++;
           }
           if (!SyncImagePixels(image))
@@ -1194,24 +1190,24 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             pixel|=(*p++ << 16);
             pixel|=(*p++ << 24);
             red=((pixel & bmp_info.red_mask) << shift.red) >> 16;
+            if (quantum_bits.red == 8)
+              red|=(red >> 8);
             green=((pixel & bmp_info.green_mask) << shift.green) >> 16;
+            if (quantum_bits.green == 8)
+              green|=(green >> 8);
             blue=((pixel & bmp_info.blue_mask) << shift.blue) >> 16;
-            opacity=((pixel & bmp_info.alpha_mask) << shift.opacity) >> 16;
+            if (quantum_bits.blue == 8)
+              blue|=(blue >> 8);
+            if (image->matte != False)
+              {
+                opacity=((pixel & bmp_info.alpha_mask) << shift.opacity) >> 16;
+                if (quantum_bits.opacity == 8)
+                  opacity|=(opacity >> 8);
+                  q->opacity=ScaleShortToQuantum(opacity);
+              }
             q->red=ScaleShortToQuantum(red);
             q->green=ScaleShortToQuantum(green);
             q->blue=ScaleShortToQuantum(blue);
-            if (image->matte)
-              q->opacity=ScaleShortToQuantum(opacity);
-#if (QuantumDepth > 8)
-            if (quantum_bits.red == 8)
-              q->red|=(q->red >> 8);
-            if (quantum_bits.green == 8)
-              q->green|=(q->green >> 8);
-            if (quantum_bits.blue == 8)
-              q->blue|=(q->blue >> 8);
-            if (quantum_bits.opacity == 8)
-              q->opacity|=(q->opacity >> 8);
-#endif
             q++;
           }
           if (!SyncImagePixels(image))
